
using Jypeli;
namespace VaistelyVille;

/// @author Juhan-Matias Rantatalo
/// @version 24.11.2023
/// <summary>
/// VäistelyVille on mukaansatempaiseva tarina sisusta, peräksiantamattomuudesta ja taidosta.
/// </summary>
public class VaistelyVille : PhysicsGame
{
    private const double NOPEUS = 200;
    private const double HYPPYNOPEUS = 750;
    private const int RUUDUN_KOKO = 40;

    private PlatformCharacter pelaaja1;
    
    private DoubleMeter alaspainLaskuri;
    private Timer aikaLaskuri;
    private IntMeter pisteLaskuri;
    
    int MARJAMAARA = 0;

    private static readonly Image[] kuvat =
        LoadImages("LeijuvaTaso", "MakeaMarja", "MonikYmpyra", "ville", "saha", "MaaTaso", "tausta1");

    public override void Begin()
    {
        LuoValikko(); 
    }
/// <summary>
/// Luodaan pelikenttä. 
/// </summary>
    private void LuoKentta() 
    {
        ClearAll();
        TileMap kentta = TileMap.FromLevelAsset("kentta1.txt");
        kentta.SetTileMethod('#', LisaaTaso);
        kentta.SetTileMethod('*', LisaaMarja);
        kentta.SetTileMethod('N', LisaaPelaaja);
        kentta.SetTileMethod('0', LisaaSaha);
        kentta.Execute(RUUDUN_KOKO, RUUDUN_KOKO);
        Level.CreateBorders();
        Level.Background.Image = kuvat[6];
        
        Gravity = new Vector(0, -1200);
        LisaaNappaimet();
        LuoAikaLaskuri();
        LuoPisteLaskuri();

        Camera.Follow(pelaaja1);
        Camera.ZoomFactor = 1.2;
        Camera.StayInLevel = true;

        MasterVolume = 0.3;
    }

/// <summary>
/// Aliohjelma lisää tasot, joilla hahmo voi hyppiä.
/// </summary>
/// <param name="paikka"></param>
/// <param name="leveys"></param>
/// <param name="korkeus"></param>
    private void LisaaTaso(Vector paikka, double leveys, double korkeus)
    {
        PhysicsObject taso = PhysicsObject.CreateStaticObject(leveys, korkeus);
        taso.Position = paikka;
        taso.Image = kuvat[5];
        Add(taso);
    }
    
/// <summary>
/// Aliohjelma lisää marjat, joita kerätään. 
/// </summary>
/// <param name="paikka"></param>
/// <param name="leveys"></param>
/// <param name="korkeus"></param>
    private void LisaaMarja(Vector paikka, double leveys, double korkeus)
    {
        PhysicsObject marja = PhysicsObject.CreateStaticObject(leveys, korkeus);
        marja.IgnoresCollisionResponse = true;
        marja.Position = paikka;
        marja.Image = kuvat[1];
        marja.Tag = "makeaMarja";
        MARJAMAARA++;
        Add(marja);
    }
    
/// <summary>
/// Aliohjelma lisää pelihahmon.
/// </summary>
/// <param name="paikka"></param>
/// <param name="leveys"></param>
/// <param name="korkeus"></param>
    private void LisaaPelaaja(Vector paikka, double leveys, double korkeus)
    {
        pelaaja1 = new PlatformCharacter(leveys, korkeus);
        pelaaja1.Position = paikka;
        pelaaja1.Mass = 4.0;
        pelaaja1.Image = kuvat[3];
        AddCollisionHandler(pelaaja1, "makeaMarja", TormaaMarjaan);
        AddCollisionHandler(pelaaja1, "saha", TormaaSahaan);
        Add(pelaaja1);
    }
    
/// <summary>
/// Aliohjelma lisää sahoja, joita tulee vältellä. 
/// </summary>
/// <param name="paikka"></param>
/// <param name="leveys"></param>
/// <param name="korkeus"></param>
    private void LisaaSaha(Vector paikka, double leveys, double korkeus)
    {
        PhysicsObject saha = PhysicsObject.CreateStaticObject(leveys, korkeus);
        saha.IgnoresCollisionResponse = true;
        saha.Position = paikka;
        saha.Image = kuvat[4];
        saha.Tag = "saha";
        Add(saha);
    }
    
/// <summary>
/// Aliohjelma lisää näppäimistön nappeihin toimintoja.
/// </summary>
    private void LisaaNappaimet()
    {
        Keyboard.Listen(Key.F1, ButtonState.Pressed, ShowControlHelp, "Näytä ohjeet");
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");

        Keyboard.Listen(Key.Left, ButtonState.Down, Liikuta, "Liikkuu vasemmalle", pelaaja1, -NOPEUS);
        Keyboard.Listen(Key.Right, ButtonState.Down, Liikuta, "Liikkuu vasemmalle", pelaaja1, NOPEUS);
        Keyboard.Listen(Key.Up, ButtonState.Pressed, Hyppaa, "Pelaaja hyppää", pelaaja1, HYPPYNOPEUS);

        ControllerOne.Listen(Button.Back, ButtonState.Pressed, Exit, "Poistu pelistä");

        ControllerOne.Listen(Button.DPadLeft, ButtonState.Down, Liikuta, "Pelaaja liikkuu vasemmalle", pelaaja1,
            -NOPEUS);
        ControllerOne.Listen(Button.DPadRight, ButtonState.Down, Liikuta, "Pelaaja liikkuu oikealle", pelaaja1, NOPEUS);
        ControllerOne.Listen(Button.A, ButtonState.Pressed, Hyppaa, "Pelaaja hyppää", pelaaja1, HYPPYNOPEUS);

        PhoneBackButton.Listen(ConfirmExit, "Lopeta peli");
    }
    
/// <summary>
/// Aliohjelma, joka tekee aikalaskurin.
/// </summary>
    private void LuoAikaLaskuri()
    {
        alaspainLaskuri = new DoubleMeter(30);
    
        aikaLaskuri = new Timer();
        aikaLaskuri.Interval = 0.1;
        aikaLaskuri.Timeout += LaskeAlaspain;
        aikaLaskuri.Start();

        Label aikaNaytto = LuoJokinLaskuri(50, -50);
        
        aikaNaytto.DecimalPlaces = 1;
        aikaNaytto.BindTo(alaspainLaskuri);
        Add(aikaNaytto);
    }
    
    /// <summary>
    /// Aliohjelma, joka laskee ajastinta alaspäin.
    /// </summary>
    void LaskeAlaspain()
    {
        alaspainLaskuri.Value -= 0.1;

        if (alaspainLaskuri.Value <= 0)
        {
            MessageDisplay.Add("Aika loppui...");
            aikaLaskuri.Stop();
            LuoValikko();
        }
    }
    
    
/// <summary>
/// Aliohjelma, joka tekee pistelaskurin.
/// </summary>
    void LuoPisteLaskuri()
    {
        pisteLaskuri = new IntMeter(0);
        Label pisteNaytto = LuoJokinLaskuri(100, -100);

        pisteNaytto.BindTo(pisteLaskuri);
        Add(pisteNaytto);
    }
    
/// <summary>
/// Aliohjelma laskureiden helpolle toteutukselle.
/// </summary>
/// <param name="x"></param>
/// <param name="y"></param>
/// <returns></returns>
    private static Label LuoJokinLaskuri(int x, int y)
    {
        
        Label naytto = new Label(); 
        naytto.X = Screen.Left + x;
        naytto.Y = Screen.Top + y;
        naytto.TextColor = Color.Black;
        naytto.Color = Color.LightBlue;
        return naytto;
    }
    
    
/// <summary>
/// Aliohjelma hahmon liikkumiselle.
/// </summary>
/// <param name="hahmo"></param>
/// <param name="nopeus"></param>
    private void Liikuta(PlatformCharacter hahmo, double nopeus)
    {
        hahmo.Walk(nopeus);
    }
    
/// <summary>
/// Aliohjelma hahmon hyppäämiselle.
/// </summary>
/// <param name="hahmo"></param>
/// <param name="nopeus"></param>
    private void Hyppaa(PlatformCharacter hahmo, double nopeus)
    {
        hahmo.Jump(nopeus);
    }
    
/// <summary>
/// Aliohjelma, joka käsittelee hahmon ja marjan törmäyksen.
/// </summary>
/// <param name="hahmo"></param>
/// <param name="makeaMarja"></param>
    private void TormaaMarjaan(PhysicsObject hahmo, PhysicsObject makeaMarja)
    {
        MessageDisplay.Add("Söit marjan!");
        pisteLaskuri.Value += 1;
        
        makeaMarja.Destroy(); //Aliohjelma vie alkuvalikkoon, jos kaikki marjat kerätään. 
        if (pisteLaskuri.Value == MARJAMAARA)
        {
            MessageDisplay.Add("Voitit pelin!");
            aikaLaskuri.Stop();
            LuoValikko();
        }
    }
    
/// <summary>
/// Aliohjelma, joka käsittelee hahmon ja esteen törmäyksen.
/// </summary>
/// <param name="hahmo"></param>
/// <param name="kohde"></param>
    public void TormaaSahaan(PhysicsObject hahmo, PhysicsObject kohde)
    {
        MessageDisplay.Add("Kuolit!");
        hahmo.Destroy();
        aikaLaskuri.Stop();
        LuoValikko();
    }
    
/// <summary>
/// Aliohjalma, joka luo valikon.
/// </summary>
    private void LuoValikko() 
    {
        MultiSelectWindow alkuValikko = new MultiSelectWindow("Väistely-Ville",
            "Aloita peli", "Lopeta");
        alkuValikko.AddItemHandler(0, LuoKentta); 
        //Aliohjelma LuoKentta ajetaan aina uudestaan, kun peli aloitetaan uudestaan.
        alkuValikko.AddItemHandler(1, Exit);
        Add(alkuValikko);
    }
}

  